@@ -82,12 +82,13 @@ from django.core.urlresolvers import reverse, get_resolver
 from django.shortcuts import get_object_or_404, redirect
 from django.utils.translation import ugettext_lazy as _
 from django.utils.http import urlquote
+from django.db.models import Model
 
 import types
 import sys
 
 from .base import *
-from .search_base import get_haystack_query
+from .search_base import get_haystack_query, MR_OD, RMR_OD
 
 #
 # Note: We understand we are playing with fire when we change the way the django view
@@ -134,6 +135,7 @@ class Category(list):
         self.name = name or cid.replace('_',' ').title()
         self.cid  = cid
         self.item = AllCategory()
+        
         # Populate items, mostly these models don't have slug columns.
         self.append( self.item )
         for item in queryset:
@@ -144,9 +146,12 @@ class Category(list):
             item.url = delay_call(view.get_url, cid, item.value)
             item.count = delay_call(view.get_count, cid, item)
 
+    def active_items(self):
+        return [ item for item in self if item.count() ]
+
     def count(self):
         """Returns the number of active items"""
-        return len(list(item for item in self if item.count())) - 1
+        return len(self.active_items()) - 1
 
     def __str__(self):
         return self.name
@@ -165,25 +170,35 @@ class CategoryListView(View):
     opts = ()
     orders = ()
     rss_view = None
+    redirect = False
+    using = 'default'
 
     def get_queryset(self, **kwargs):
-        self.query = self.request.GET.get('q', None)
+        self.query = self.get_value('q', None)
         def _get(cat):
-            return hasattr(self.model, cat.cid) and cat.item or cat.item.value
+            field = getattr(self.model, cat.cid, None)
+            if field and (not isinstance(field, (MR_OD, RMR_OD)) or not self.query):
+                return cat.item
+            return cat.item.value
 
         if self.query:
-            queryset = get_haystack_query(self.model, self.query)
+            queryset = get_haystack_query(self.query, using=self.using,
+                models=(self.model,))
         else:
             queryset = self.model._default_manager.all()
         filters = dict((cat.cid, _get(cat)) for cat in self.get_cats() if cat)
 
         filters.update(clean_dict(dict(self.get_opts())))
         filters.update(kwargs)
+        sys.stderr.write("Fields: %s\n" % str(filters))
         qs = queryset.filter(**clean_dict(filters, {'True':True,'False':False}))
-        try:
-            return qs.order_by(self.get_value('order'))
-        except:
-            return qs
+        ob = self.get_value('order')
+        if ob:
+            try:
+                return qs.order_by(ob)
+            except:
+                pass
+        return qs
 
     def get_url(self, cid=None, value=None, view=None, exclude=None):
         kwargs = self.kwargs.copy()
@@ -224,8 +239,8 @@ class CategoryListView(View):
             item = None
         return self.get_queryset(**{cid: item}).count()
 
-    def get_value(self, cid):
-        return self.kwargs.get(cid, self.request.GET.get(cid, None))
+    def get_value(self, cid, default=None):
+        return self.kwargs.get(cid, self.request.GET.get(cid, default))
 
     @cached
     def get_opts(self):
@@ -280,20 +295,22 @@ class CategoryListView(View):
 
     def get_cat(self, cid, name, model=None):
         # We could move this into Category class XXX
-        if hasattr(self.model, cid):
-            qs = getattr(self.model, cid).get_query_set()
+        field = getattr(self.model, cid, None)
+        if field and hasattr(field, 'get_query_set'):
+            qs = field.get_query_set()
         elif model:
-            qs = model.objects.all()
+            if isinstance(model, Model):
+                qs = model.objects.all()
+            elif isinstance(model, basestring):
+                qs = getattr(self, model)()
         else:
             raise KeyError(("The field '%s' isn't a ForeignKey, please add "
                            "the linked model for this category.") % cid)
         return Category(self, qs, cid, name)
 
     def get(self, *args, **kwargs):
-        # XXX There is going to be subtle problems here between
-        # When we want items to redirect and when we don't.
         qs = self.get_queryset()
-        if not self.query and qs.count() == 1:
+        if self.redirect and not self.query and qs.count() == 1:
             item = qs.get()
             if hasattr(item, 'get_absolute_url'):
                 return redirect( item.get_absolute_url() )
@@ -309,7 +326,7 @@ class CategoryListView(View):
         if not hasattr(self.model, 'object'):
             self.model.object = lambda self: self
 
-        data['query'] = self.query
+        data['query'] = self.get_value('q')
         data['orders'] = self.get_orders()
         data['categories'] = self.get_cats()
         data.update(((cat.cid, cat.item) for cat in self.get_cats() if cat))
@@ -321,9 +338,9 @@ class CategoryListView(View):
         return data
 
     def get_orders(self):
-        order = self.get_value('order')
+        order = self.get_value('order', self.orders[0][0])
         for (o, label) in self.orders:
-            yield { 'id': o, 'name': label, 'down': '-' == order[0],
+            yield { 'id': o, 'name': label, 'down': '-' == (order or '*')[0],
                 'active': order.strip('-') == o.strip('-'),
                 'url': self.get_url('order', reverse_order(o, o == order)) }
 
